/*
 * listen.js v0.3.0 https://github.com/mantoni/listen.js
 *
 * @generator nomo.js v0.1.5
 */
(function (modules) {
  'use strict';
  var instances = {};
  var delegate = window.require;

  function require(name) {
    if (!instances.hasOwnProperty(name)) {
      var factory = modules[name];
      if (!factory) {
        if (delegate) {
          return instances[name] = { exports: delegate(name) };
        }
        throw new Error('Unknown module: ' + name);
      }
      var exports = {};
      instances[name] = { exports: exports };
      factory(instances[name], exports, require);
    }
    return instances[name].exports;
  }

  require.define = function (name, factory) {
    if (modules[name]) {
      throw new Error('Module "' + name + '" already defined.');
    }
    modules[name] = factory;
  };

  window.listen = require('lib/listen');

}({
  'lib/listen': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012-2013 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    var ErrorList    = require('lib/error-list');
    var TimeoutError = require('lib/timeout-error');

    function listen(initialValues) {
      var values;
      if (arguments.length) {
        if (Object.prototype.toString.call(initialValues) !== '[object Array]') {
          throw new TypeError('Array expected');
        }
        values = Array.prototype.slice.call(initialValues);
      } else {
        values = [];
      }

      var offset = values.length;
      var count  = 0;
      var called = 0;
      var errList;
      var handler;

      function resolve(fn) {
        if (errList) {
          fn(errList.length === 1 ? errList[0] : new ErrorList(errList));
        } else {
          fn(null, values);
        }
      }

      function assertUnresolved() {
        if (handler) {
          throw new Error("Cannot be called after then");
        }
      }

      function pushError(e) {
        if (!errList) {
          errList = [];
        }
        if (e.name === 'ErrorList') {
          Array.prototype.push.apply(errList, e.errors);
        } else {
          errList.push(e);
        }
      }

      function withTimeout(delegate, timeout) {
        var timer = setTimeout(function () {
          delegate(new TimeoutError());
          delegate = null;
        }, timeout);
        return function (err, value) {
          if (delegate) {
            clearTimeout(timer);
            delegate(err, value);
          }
        };
      }

      function listener(fn, timeout) {
        if (typeof fn === 'number') {
          timeout = fn;
          fn      = null;
        }
        assertUnresolved();
        var index = offset + count++;
        var callback = function (err, value) {
          if (fn) {
            try {
              fn(err, value);
            } catch (e) {
              if (e !== err) {
                pushError(e);
              }
            }
          }
          if (err) {
            pushError(err);
          }
          if (value !== undefined) {
            values[index] = value;
          }
          if (++called === count && handler) {
            resolve(handler);
          }
        };
        return timeout ? withTimeout(callback, timeout) : callback;
      }

      listener.push = function push(value) {
        assertUnresolved();
        values[offset + count] = value;
        offset++;
      };

      listener.err = function err(e) {
        assertUnresolved();
        pushError(e);
      };

      listener.then = function then(fn) {
        if (handler) {
          throw new Error('Cannot be called more than once');
        }
        if (typeof fn !== 'function') {
          throw new TypeError('Function expected');
        }
        handler = fn;
        if (called === count) {
          resolve(fn);
        }
      };

      return listener;
    }

    listen.ErrorList = ErrorList;

    module.exports = listen;

  },
  'lib/error-list': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function ErrorList(errors) {
      this.name     = 'ErrorList';
      var sep       = '\n  - ';
      this.message  = 'Multiple callbacks err\'d:' + sep + errors.join(sep);
      this.errors   = errors;
    }

    ErrorList.prototype = Error.prototype;

    module.exports = ErrorList;

  },
  'lib/timeout-error': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function TimeoutError() {
      this.name = 'TimeoutError';
    }

    TimeoutError.prototype = Error.prototype;

    module.exports = TimeoutError;

  }
}));
